#! /usr/bin/python3

import os
import argparse
import socket
from os.path import join
import json
from time import time

from evdev import InputDevice, ecodes as e
import paho.mqtt.client as mqtt
import yaml

version = "0.2.0"

def xdg_config_dir():
    config_home = os.getenv("XDG_CONFIG_HOME",
                            os.path.expanduser("~/.config/"))
    return os.path.join(config_home, "evdev2mqtt")

def bind_layer(layers, key):
    layer = layers["_"].copy()
    layer.update(layers[key])

    return layer

def loop(dev_path, mqtt_client, init, layers):
    dev = InputDevice(dev_path)
    dev.grab()
    print("Grabbing", dev)
    # print(dev.capabilities(verbose=True))

    if init in layers:
        binding = bind_layer(layers, init)
    else:
        binding = bind_layer(layers, "_")

    for event in dev.read_loop():
        if event.value != 1: # down
            continue

        key = e.KEY[event.code]

        # KEY_MUTE has an alias named KEY_MIN_INTERESTING so when
        # KEY_MUTE is pressed, the following list is returned:
        # ['KEY_MIN_INTERESTING', 'KEY_MUTE']
        if type(key) is list and 'KEY_MUTE' in key:
            key = 'KEY_MUTE'

        if key in layers:
            binding = bind_layer(layers, key)

        if key not in binding:
            print(key, None)
            continue

        action = binding[key]
        messages = []
        if type(action) is str:
            messages.append((action, {}))
        elif type(action) is dict:
            for topic, data in action.items():
                if data is None:
                    messages.append((topic, {}))
                else:
                    messages.append((topic, action[topic]))

        for (topic, data) in messages:
            print(key, topic, data)
            mqtt_client.publish(topic, json.dumps(data))

if __name__=="__main__":
    parser = argparse.ArgumentParser(description='evdev2mqtt -- Evdev to MQTT Gateway')
    parser.add_argument('--version', action='version', version='%(prog)s ' + version)
    parser.add_argument("-b", "--host",
                        dest="host",
                        default="localhost",
                        help="MQTT broker address")
    parser.add_argument("-c", "--config",
                        dest="config",
                        default=join(xdg_config_dir(), "configuration.yml"),
                        help="Configuration file")
    parser.add_argument("-d", "--device",
                        dest="device",
                        default="/dev/input/ir",
                        help="Path to the input device")
    parser.add_argument("--init",
                        dest="init",
                        default="_",
                        help="Layer initially selected")
    args = parser.parse_args()

    with open(args.config) as f:
        config = yaml.safe_load(f)

    client = mqtt.Client()
    client.connect(args.host)
    client.loop_start()

    loop(args.device, client, args.init, config)

